/*****************************************************************************
*
*   Analog Devices, Inc.
*   ADSP-BF537 EZ-Kit Lite
*   Networked Audio Client
*
******************************************************************************
*
*   File        :   audio.c
*
*   Description :   This file contains routines for the audio application
*                   thread.
*
******************************************************************************
*   Revision History
******************************************************************************
*   date        rev     by      description
*-----------------------------------------------------------------------------
*   02.27.07    01.02h  KCK     Original release.
*****************************************************************************/

#include <cdefBF537.h>
#include <lwip/sockets.h>
#include <lwip/inet.h>
#include <services/services.h>      // system service includes
#include <drivers/adi_dev.h>        // device manager includes
#include <drivers/sport/adi_sport.h>// sport driver includes

#include "system.h"
#include "audio.h"
#include "boot.h"

int     audioSocketID;
struct  sockaddr_in clientAudioSocketAddress;
struct  sockaddr_in serverAudioSocketAddress;
volatile int audioPacketCount = 0;

ADI_DEV_1D_BUFFER   audioBuffer;                                        // the buffer descriptor
#pragma align 4
section ("L1_data") char audioData[AUDIO_BUFFER_COUNT][AUDIO_BUFFER_LEN];    // the actual data
#pragma align 4
/*section ("L1_data")*/ char audioMuteData[AUDIO_BUFFER_LEN * AUDIO_BUFFER_COUNT];

#pragma align 4
section ("L1_data") char audioRecvBuffer[AUDIO_RECV_BUFFER_LEN];

int audioDataSize[AUDIO_BUFFER_COUNT];
volatile int audioDataInputIndex;
volatile int audioDataOutputIndex;

volatile u32 sportTCLKDIVAdjust;
volatile u32 sportTCLKDIVFast;
volatile u32 sportTCLKDIVSlow;
volatile u32 sportTCLKDIVDefault;

ADI_DEV_DEVICE_HANDLE   audioOutHandle;   // handle to the audio output driver


static VDK_ThreadCreationBlock audioFlowTCB = 
    {
    kaudioflow,         // template ID
    (VDK_ThreadID)0,    // thread ID
    0,                  // thread stack size
    (VDK_Priority)0,    // thread priority
    0,                  // user data pointer
    0                   // thread template
    };


// thread creation block to send keep alive messages over the network
static VDK_ThreadCreationBlock keepAliveTCB = 
    {
    kkeepalive,         // template ID
    (VDK_ThreadID)0,    // thread ID
    0,                  // thread stack size
    (VDK_Priority)0,    // thread priority
    0,                  // user data pointer
    0                   // thread template
    };


int
AudioNetworkConnect(void)
    {

    // create an Audio UPD socket (SOCK_DGRAM) with Internet Protocol Family (PF_INET)
    audioSocketID = socket(PF_INET, SOCK_DGRAM, 0);

    if ( audioSocketID < 0 )
        {
        #ifdef DEBUG_PRINT_ENABLE
        printf("socket call failed");
        #endif
        exit(0);
        }

    memset(&clientAudioSocketAddress, 0, sizeof(struct sockaddr_in));
    clientAudioSocketAddress.sin_family = AF_INET;
    clientAudioSocketAddress.sin_addr.s_addr = inet_addr(CLIENT_IP_ADDR);
    clientAudioSocketAddress.sin_port = htons(CLIENT_AUDIO_PORT);


    // Bind the socket to the port CLIENT_AUDIO_PORT and to the current
    // machines IP address (Its defined by CLIENT_IP_ADDR). 
    // Once bind is successful for UDP sockets application can operate
    // on the socket descriptor for sending or receiving data.

    if (bind(audioSocketID, (struct sockaddr *)&clientAudioSocketAddress, sizeof(struct sockaddr_in)) < 0)
        {
        #ifdef DEBUG_PRINT_ENABLE
        printf("Bind to Port Number %d ,IP address %s failed\n",CLIENT_AUDIO_PORT, CLIENT_IP_ADDR);
        #endif
        close(audioSocketID);
        exit(1);
        }

    memset(&serverAudioSocketAddress, 0, sizeof(struct sockaddr_in));
    serverAudioSocketAddress.sin_family = AF_INET;
    serverAudioSocketAddress.sin_addr.s_addr = inet_addr(SERVER_IP_ADDR);
    serverAudioSocketAddress.sin_port = htons(SERVER_AUDIO_PORT);

    if (connect(audioSocketID, (struct sockaddr *)&serverAudioSocketAddress, sizeof(struct sockaddr) ) < 0)
        {
        #ifdef DEBUG_PRINT_ENABLE
        printf("Connect failed\n"); 
        #endif
        close(audioSocketID);
        return(-1);
        }
    return(0);
    }


//  made this a distinct routine in case audio processing needs to be added
//  in the future
int
RecvAudio(char *buf)
    {
    int i;
    int returnVal;

    returnVal = recv(audioSocketID, audioRecvBuffer, AUDIO_RECV_BUFFER_LEN, 0);

    if (returnVal > 0)
        {

        for (i = 0; i < AUDIO_RECV_BUFFER_LEN;)
            {
            // stereo channel 0
            *buf++ = audioRecvBuffer[i++];
            *buf++ = audioRecvBuffer[i++];

            // stereo channel 1
            *buf++ = audioRecvBuffer[i++];
            *buf++ = audioRecvBuffer[i++];
            }
        }

    return returnVal;
    }



void
audio_RunFunction(void **inPtr)
    {
    u32                     result;                 // results of the function calls

    *pPORTFIO_SET   = 0x0040; // set LED 1

    if(AudioNetworkConnect() > -1)
        {

        #ifdef DEBUG_PRINT_ENABLE
        printf("Audio connection established\n");
        #endif

        // spawn keep alive thread
        result = VDK_CreateThreadEx ( &keepAliveTCB );

        AudioOutputInit();


        // fill half of the buffers before starting DAC output
        for (audioDataInputIndex = 0; audioDataInputIndex < AUDIO_BUFFER_MID; ++audioDataInputIndex)
            {
            audioDataSize[audioDataInputIndex] = RecvAudio(audioData[audioDataInputIndex]);
            *pPORTFIO_TOGGLE = 0x0800; // toggle LED 6
            ++audioPacketCount;
            }

#ifdef DEBUG_PRINT_ENABLE
        printf("Starting audio output\n");
#endif

        // start DAC output
        AudioOutputStart();

        // finishing filling buffers
        for (audioDataInputIndex = AUDIO_BUFFER_MID; audioDataInputIndex < AUDIO_BUFFER_COUNT; ++audioDataInputIndex)
            {
            audioDataSize[audioDataInputIndex] = RecvAudio(audioData[audioDataInputIndex]);
            *pPORTFIO_TOGGLE = 0x0800; // toggle LED 6
            ++audioPacketCount;
            }

        // spawn audio flow control thread
        result = VDK_CreateThreadEx ( &audioFlowTCB );


        // steady state
        while (1)
            {
            *pPORTFIO_TOGGLE = 0x0800; // toggle LED 6
            
            
            for (audioDataInputIndex = 0; audioDataInputIndex < AUDIO_BUFFER_COUNT; ++audioDataInputIndex)
                {
                audioDataSize[audioDataInputIndex] = RecvAudio(audioData[audioDataInputIndex]);
                ++audioPacketCount;
                }
            }
        }
    }



int
audio_ErrorFunction(void **inPtr)
    {
    VDK_DestroyThread(VDK_GetThreadID(), false);
    return 0;
    }

void
audio_InitFunction(void **inPtr, VDK_ThreadCreationBlock *pTCB)
    {
    }

void
audio_DestroyFunction(void **inPtr)
    {
    }






static void AudioOutCallback(void *appHandle,
    u32  event,
    void *pArg)
    {
    int index;
    u32 result;
    ADI_DEV_1D_BUFFER *buffer;


    switch (event)
        {

        case ADI_DEV_EVENT_BUFFER_PROCESSED :


            // toggle TX LED
            *pPORTFIO_TOGGLE = 0x0400; // toggle LED 5

            buffer = (ADI_DEV_1D_BUFFER *)pArg;


            // if no new packets have arrived we need to mute the audio
            if (audioPacketCount == 0)
                {
                buffer->Data = audioMuteData;   // Start address of mute buffer
                }
            else
                {
                buffer->Data = audioData[0];    // Start address of data buffer
                }

            // requeue the buffer
            result = adi_dev_Write(audioOutHandle, ADI_DEV_1D, (ADI_DEV_BUFFER *)buffer);

            audioPacketCount = 0;


            // buffer flow control management
            // the I2S clock updates are calculated here and applied in the audioflow thread

            // if output is getting ahead of input, slow down audio output
            if ((audioDataInputIndex <= AUDIO_BUFFER_LO_THR))
                {
                sportTCLKDIVAdjust = sportTCLKDIVSlow;
                *pPORTFIO_SET = 0x0200; // set LED 4
                }

            // if input is getting ahead of output, speed up audio output
            else if (audioDataInputIndex >= AUDIO_BUFFER_HI_THR)
                {
                sportTCLKDIVAdjust = sportTCLKDIVFast;
                *pPORTFIO_SET = 0x0200; // set LED 4
                }
            // input and output are keeping pace
            else
                {
                sportTCLKDIVAdjust = sportTCLKDIVDefault;
                *pPORTFIO_CLEAR = 0x0200; // set LED 4
                }

        break;
        }
    }



void 
AudioOutputInit()
    {
    int i;
    u32 result;

    // prepare buffer for audio output driver to store the data
    audioBuffer.Data                 = audioData[0];
    audioBuffer.ElementCount         = (AUDIO_ELEMENT_COUNT * AUDIO_BUFFER_COUNT);
    audioBuffer.ElementWidth         = AUDIO_ELEMENT_WIDTH;
    audioBuffer.CallbackParameter    = &audioBuffer;
    audioBuffer.ProcessedFlag        = FALSE;
    audioBuffer.pNext                = (ADI_DEV_1D_BUFFER *)NULL;



    //  initialize a zeroed buffer for muting purposes
    for (i = 0; i < (AUDIO_BUFFER_LEN * AUDIO_BUFFER_COUNT); ++i)
        {
        audioMuteData[i] = 0;
        }

    // open the audio output driver for inbound data flow
    result = adi_dev_Open(
        devManagerHandle,           // device manager handle
        &ADISPORTEntryPoint,        // entry point
        0,                          // device number
        &audioOutHandle,            // client handle
        &audioOutHandle,            // device handle
        ADI_DEV_DIRECTION_OUTBOUND, // direction
        dmaManagerHandle,           // DMA Handle
        NULL,                       // DCB Handle
        AudioOutCallback);          // callback routine

    sportTCLKDIVDefault = AUDIO_TCLKDIV_DEFAULT;
    sportTCLKDIVFast    = AUDIO_TCLKDIV_FAST;
    sportTCLKDIVSlow    = AUDIO_TCLKDIV_SLOW;
    sportTCLKDIVAdjust  = AUDIO_TCLKDIV_DEFAULT;

    ADI_DEV_CMD_VALUE_PAIR audioInConfig [] = {
        { ADI_DEV_CMD_SET_DATAFLOW_METHOD,      (void *)ADI_DEV_MODE_CHAINED            },
        { ADI_DEV_CMD_SET_STREAMING,            (void *)FALSE                           },
        { ADI_SPORT_CMD_SET_TCR1,               (void *)(TFSR | LTFS | TCKFE | DITFS | ITCLK | ITFS)},
        { ADI_SPORT_CMD_SET_TCR2,               (void *)(SLEN_16 | TSFSE)               },
        { ADI_SPORT_CMD_SET_TCLKDIV,            (void *)AUDIO_TCLKDIV_DEFAULT                       },
        { ADI_SPORT_CMD_SET_TFSDIV,             (void *)AUDIO_TFSDIV_DEFAULT                        },
        { ADI_DEV_CMD_END,                      NULL                                    }
    };

    result = (adi_dev_Control(audioOutHandle, ADI_DEV_CMD_TABLE, audioInConfig));

    // give the buffers to audio output driver
    result = adi_dev_Write(audioOutHandle, ADI_DEV_1D, (ADI_DEV_BUFFER *)&audioBuffer);
    }


void 
AudioOutputStart()
    {
    u32 result;

    // enable data flow on audio output driver
    result = adi_dev_Control(audioOutHandle, ADI_DEV_CMD_SET_DATAFLOW, (void *)TRUE);
    }


