/*****************************************************************************
*
*   Analog Devices, Inc.
*   ADSP-BF537 EZ-Kit Lite
*   Networked Audio Server
*
******************************************************************************
*
*   File        :   boot.c
*
*   Description :   This file contains boot routines.
*
******************************************************************************
*   Revision History
******************************************************************************
*   date        rev     by      description
*-----------------------------------------------------------------------------
*   02.27.07    01.02h  KCK     Original release.
*****************************************************************************/
#include "system.h"
#include "boot.h"

#include <cdefBF537.h>
#include <lwip/cglobals.h>
#include <lwip/kernel_abs.h>
#include <lwip/ADI_TOOLS_IOEThreadType.h>
#include <lwip/sockets.h>
#include <lwip/inet.h>
#include <lwip/ADI_ETHER_BF537.h>


#include <sys/exception.h>
#include <services/services.h>
#include <drivers/adi_dev.h>
#include <stdio.h>
#include <string.h>

/* Size of the memory block to allocate to the stack */
#define ETHER_STACK_SIZE    1024 * 1024 * 2


/* Initializes the Stack */
int system_init(void);

/*
 *  Global static data for various SSL managers 
 */
static ADI_INT_IMASK    imask_storage;

#pragma alignment_region (4)

static u8 devManagerData[ADI_DEV_BASE_MEMORY + (ADI_DEV_DEVICE_MEMORY * 16)];
static u8 intManagerData[                      (ADI_INT_SECONDARY_MEMORY * 16)];
static u8 dcbManagerData[ADI_DCB_QUEUE_SIZE +  (ADI_DCB_ENTRY_SIZE) * 20];
static u8 dmaManagerData[ADI_DMA_BASE_MEMORY + (ADI_DMA_CHANNEL_MEMORY * 20)];

#pragma alignment_region_end

ADI_DCB_HANDLE          dcbManagerHandle;   // Deferred Callback Manager handle
ADI_DMA_MANAGER_HANDLE  dmaManagerHandle;   // DMA Manager handle
ADI_DEV_MANAGER_HANDLE  devManagerHandle;   // Device Manager handle
ADI_DEV_DEVICE_HANDLE   lanHandle;          // LAN handle



static char hwaddr[6];
#define NO_RCVES 64
#define NO_XMITS 64

#pragma alignment_region (32)
char BaseMemSize[ADI_ETHER_MEM_BF537_BASE_SIZE];
char MemRcve[NO_RCVES*ADI_ETHER_MEM_BF537_PER_RECV];
char MemXmit[NO_XMITS*ADI_ETHER_MEM_BF537_PER_XMIT];
#pragma alignment_region_end

static int p_period = 1;

ADI_ETHER_SUPPLY_MEM memtable = {
    MemRcve,sizeof(MemRcve),0,
    MemXmit,sizeof(MemXmit),0,
    BaseMemSize,sizeof(BaseMemSize)};


void
boot_RunFunction(void **inPtr)
    {

    if(system_init() == -1)
        {
        #ifdef DEBUG_PRINT_ENABLE
        printf("Failed to initialize system\n");
        #endif
        return;
        }


    // start stack
    start_stack();


    // spawn audio thread
    {
    VDK_ThreadCreationBlock TCB = 
        {
        kaudio,
        (VDK_ThreadID)0,
        0,
        (VDK_Priority)0,
        0,
        0
        };
            
    if ( UINT_MAX == VDK_CreateThreadEx ( &TCB ) )
        {
        #ifdef DEBUG_PRINT_ENABLE
        printf ( "Call to VDK_CreateThreadEx() failed.\n" );
        #endif
        abort();
        }
    }


    }

int
boot_ErrorFunction(void **inPtr)
    {

    VDK_DestroyThread(VDK_GetThreadID(), false);
    return 0;
    }

void
boot_InitFunction(void **inPtr, VDK_ThreadCreationBlock *pTCB)
    {
    }

void
boot_DestroyFunction(void **inPtr)
    {
    }

u16 phyregs[32];

static u32 ssl_cclk, ssl_sclk, ssl_fvco;

int system_init()
    {

    u32 result;

    int phyAddress;

    u32 responseCount;
    u32 critical_reg;
    char *ether_stack_block;
    const unsigned int mac_address_in_flash = 0x203F0000;

    // initialize power settings
    ADI_PWR_COMMAND_PAIR powerCmdTable[] = { 
        { ADI_PWR_CMD_SET_PROC_VARIANT,(void*)ADI_PWR_PROC_BF537SKBC1600 }, // 600MHz variant 
        { ADI_PWR_CMD_SET_PACKAGE,     (void*)ADI_PWR_PACKAGE_MBGA       }, // in MBGA packaging, as on all EZ-KITS
        { ADI_PWR_CMD_SET_VDDEXT,      (void*)ADI_PWR_VDDEXT_330         }, // external voltage supplied to the voltage regulator is 3.3V
        { ADI_PWR_CMD_SET_CLKIN,       (void*)ADI_PWR_CLKIN_EZKIT_BF537  }, // the CLKIN frequency 25 MHz
        { ADI_PWR_CMD_END,             0                                 } 
    }; 
    result = adi_pwr_Init( powerCmdTable );

    #ifdef DEBUG_PRINT_ENABLE
    DEBUG_PRINT("Failed to initialize Power Service\n",result != ADI_DEV_RESULT_SUCCESS);
    #endif

    adi_pwr_SetFreq( 
        600000000,          // cclk, 
        133000000,          // sclk, 
        ADI_PWR_DF_NONE);   // let routine choose input multiplier

    adi_pwr_GetFreq (&ssl_cclk, &ssl_sclk, &ssl_fvco);
    
    VDK_SetClockFrequency(ssl_cclk/1000000);

    // initialize the EBIU settings
    ADI_EBIU_RESULT EBIUResult;


    static ADI_EBIU_SDRAM_BANK_VALUE bank_size =
        {0, ADI_EBIU_SDRAM_BANK_32MB};          // 32 MB 





#define DO_NOT_CHANGE_MMR_SETTINGS 0


    ADI_EBIU_COMMAND_PAIR sdramConfig[] = {
        { ADI_EBIU_CMD_SET_EZKIT,               (void*)ADI_EBIU_EZKIT_BF537 },
        { ADI_EBIU_CMD_END,                     0                           }

    };


    EBIUResult = adi_ebiu_Init( sdramConfig, DO_NOT_CHANGE_MMR_SETTINGS );


    DEBUG_PRINT("Failed to initialize EBIU\n",EBIUResult != ADI_DEV_RESULT_SUCCESS);

    // set traffic control to none and DMA higher priority than core
    *pDMA_TC_PER    = 0x0000;
    *pEBIU_AMGCTL   = 0x01ff;


    // initialize the interrupt manager

    result = adi_int_Init(
       intManagerData,
       sizeof(intManagerData),
       &responseCount,
       &critical_reg);

    #ifdef DEBUG_PRINT_ENABLE
    DEBUG_PRINT("Failed to initialize Interrupt Manager\n",result != ADI_DEV_RESULT_SUCCESS);
    #endif


    // initialize the device manager

    result = adi_dev_Init(
        devManagerData,
        sizeof(devManagerData),
        &responseCount,
        &devManagerHandle,
        &imask_storage);

    #ifdef DEBUG_PRINT_ENABLE
    DEBUG_PRINT("Failed to initialize Device Manager\n",result != ADI_DEV_RESULT_SUCCESS);
    #endif

    // initialize the deferred callback manager
    result = adi_dcb_Init(
        &dcbManagerData[0], 
        ADI_DCB_QUEUE_SIZE, 
        &responseCount, 
        NULL);

    result = adi_dcb_Open(
        14, 
        &dcbManagerData[ADI_DCB_QUEUE_SIZE], 
        (ADI_DCB_ENTRY_SIZE)*4, 
        &responseCount, 
        &dcbManagerHandle);

    #ifdef DEBUG_PRINT_ENABLE
    DEBUG_PRINT("Failed to initialize Deferred Callback Manager\n",result != ADI_DEV_RESULT_SUCCESS);
    #endif

    // initialize the DMA manager
    result = adi_dma_Init(
        dmaManagerData, 
        sizeof(dmaManagerData), 
        &responseCount, 
        &dmaManagerHandle, 
        NULL);

    #ifdef DEBUG_PRINT_ENABLE
    DEBUG_PRINT("Failed to initialize DMA Manager\n",result != ADI_DEV_RESULT_SUCCESS);
    #endif

    
    // initialize ports
    *pPORTFIO_DIR   = 0x0FC0;
    *pPORTFIO_CLEAR = 0x0FC0; // clear all LEDs


    // initialize the kernel

    ker_init((void*)0);

    // set thread type for the stack threads

    ker_set_auxdata((void*)kADI_TOOLS_IOEThreadType);



    // open lan-device

    result = adi_dev_Open(
       devManagerHandle,
       &ADI_ETHER_BF537_Entrypoint,
       0,
       NULL,
       &lanHandle,
       ADI_DEV_DIRECTION_BIDIRECTIONAL,
       NULL,
       NULL,
       (ADI_DCB_CALLBACK_FN)stack_callback_handler);

    #ifdef DEBUG_PRINT_ENABLE
    DEBUG_PRINT("Failed to open the lan-device\n",result != ADI_DEV_RESULT_SUCCESS);
    #endif

    /* set the services with in stack */

    set_pli_services(1,&lanHandle);


    /*
     * Read the EZ-KIT Lite's assigned MAC address, found at address 0x203F0000.
     * 
     */

    memcpy ( &hwaddr, (unsigned char *) mac_address_in_flash, sizeof ( hwaddr ) );

    result = adi_dev_Control(
                              lanHandle,
                              ADI_ETHER_CMD_SET_MAC_ADDR,
                              (void*)&hwaddr);
                          
    #ifdef DEBUG_PRINT_ENABLE
    DEBUG_PRINT("Failed set MAC address\n",result != ADI_DEV_RESULT_SUCCESS);
    #endif

    // supply some memory for the driver

    result = adi_dev_Control(
                              lanHandle,
                              ADI_ETHER_CMD_SUPPLY_MEM,
                              &memtable);

    #ifdef DEBUG_PRINT_ENABLE
    DEBUG_PRINT("Failed to supply memory to driver\n",result != ADI_DEV_RESULT_SUCCESS);
    #endif

    result = adi_dev_Control(
                              lanHandle,
                              ADI_DEV_CMD_SET_DATAFLOW_METHOD,
                              (void*)TRUE);


    /* Initialze the stack with user specified configuration priority -3 and
     * poll period of p_period msec.  The stack is allocated a memory buffer as well.
     */

    ether_stack_block = (char *) malloc ( ETHER_STACK_SIZE );

    #ifdef DEBUG_PRINT_ENABLE
    DEBUG_PRINT("Failed to malloc Ethernet Stack\n",!ether_stack_block);
    #endif

    init_stack ( 3, p_period, ETHER_STACK_SIZE, ether_stack_block );


    // start the MAC
    #ifdef PHY_INT5200      // PHY_INT5200 specific processing (set in system.h)
    result = adi_dev_Control (
        lanHandle, 
        ADI_ETHER_CMD_BF537_SET_PHY_ADDR, 
        (void *)0);
    #else
    result = adi_dev_Control (
        lanHandle, 
        ADI_ETHER_CMD_BF537_SET_PHY_ADDR, 
        (void *)1);
    #endif

    result = adi_dev_Control (
        lanHandle,
        ADI_ETHER_CMD_START,
        NULL);

    #ifdef DEBUG_PRINT_ENABLE
    DEBUG_PRINT("Failed to start the LAN Driver\n",result != ADI_DEV_RESULT_SUCCESS);
    #endif


    // read the PHY controller registers
    result = adi_dev_Control(
        lanHandle, 
        ADI_ETHER_CMD_GET_PHY_REGS, 
        phyregs);


    #ifdef DEBUG_PRINT_ENABLE
    DEBUG_PRINT("PHY Controller has failed and the board needs power cycled\n", phyregs[1]==0xFFFF);
    #endif

    // wait for the link to be up

    #ifdef PHY_INT5200      // PHY_INT5200 specific processing (set in system.h)
    if ( (phyregs[1] & 0x2) == 0)
    #else
    if ( (phyregs[1] & 0x4) == 0)
    #endif
        {
        #ifdef DEBUG_PRINT_ENABLE
        printf("Waiting for the link to be established\n");
        #endif
        #ifdef PHY_INT5200      // PHY_INT5200 specific processing (set in system.h)
        while ( (phyregs[1] & 0x2) == 0)
        #else
        while ( (phyregs[1] & 0x4) == 0)
        #endif
            {
            adi_dev_Control(lanHandle,ADI_ETHER_CMD_GET_PHY_REGS,phyregs);
            }
        }

    #ifdef DEBUG_PRINT_ENABLE
    printf("Link established\n");
    #endif

    return 1;
    }
